package database;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import global.GlobalData;
import global.GlobalData.CampoEnum;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import tratamientoCadenas.Cadena;

/**
 * La clase IntroducirDatos est enfocada al almacenamiento de datos 
 * en la BD creada para Goodwave a partir de un fichero XML con el formato siguiente:
 * <ConjuntoDatos>
 * 	<Tupla>
 * 		<Variable>Poblacin</Variable>
 * 		<AditividadTemporal>Promedio</AditividadTemporal>	//valor en el conjunto {Aditivo, Promedio, Maximo, Minimo}
 * 		<AditividadEspacial>Aditivo</AditividadEspacial>	//valor en el conjunto {Aditivo, Promedio, Maximo, Minimo}
 * 		<Frecuencia>Ano</Frecuencia>	//valor en el conjunto {dia, semana, quincena, mes, ano
 * 		<Fecha>2008</Fecha>		//en los formatos: dd-mm-aa, dd-mm-aaaa, mm-aa, mm-aaaa, aaaa (donde '-' puede ser tambin '/')
 * 		<Unidades>Personas</Unidades>
 * 		<Lugar>Ceuta</Lugar>
 * 		<Nivel>Municipio</Nivel>	//para el caso de Espaa, valor en el conjunto {"Municipio", "Provincia", "Autonomia", "Pais"}
 * 									//para otros paises, valor en el conjunto especificado por el administrador que introduzca
 * 									//en la BD la jerarquia de regiones de dicho pais
 * 		<TipoDato>INT</TipoDato>	//valor en el conjunto {int, float, string, bool}
 * 		<NaturalezaDato>Demografia</NaturalezaDato>
 * 		<Valor>"77389.0"</Valor>
 * 		<OrigenDatos>INE</OrigenDatos>	//deber ser un cdigo vlido de organismo proveedor de datos
 * 	</Tupla>
 * 	<Tupla>
 * 		...
 * 	</Tupla>
 * 	...
 * </ConjuntoDatos>
 * 
 * @author Francisco Jos Garca Rodrigo
 * */
public class IntroducirDatos {
	
	/**
	 * Convierte a <b>Document</b> un <b>String</b> con el contenido de un fichero XML
	 * @param xmlString <b>String</b> con el contenido correcto de un fichero XML
	 * @return Un objeto <b>Document</b> con el contenido XML
	 * 
	 * @pre <i>xmlString</i> debe tener el formato indicado en la cabecera de la clase 
	 * */
	//TODO colocar en la clase correcta!!
	public static Document String2Document(String xmlString){ 
		
		Document document = null;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		
		try {
			
			builder = factory.newDocumentBuilder();
			document = builder.parse(new InputSource(new StringReader(xmlString)));
			
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return document;
	}
	
	/**
	 * Introduce en la BD de goodwave los datos almacenados por el <b>Document</b> <i>xmldoc</i>
	 * @param xmldoc <b>Document</b> con el contenido correcto de un fichero XML
	 *  
	 * @pre <i>xmldoc</i> debe tener el formato indicado en la cabecera de la clase 
	 * */
	public static void introducirEnBDcamposString (Document xmldoc){
		
		Connection conn = null;
		String [] infoVariable = new String [GlobalData.NUM_ATRIBS_VARIABLES_SIN_ID];
		String fecha = null;
		String lugar = null;
		String valor = null;
		String nivel = null;
		
		try
		{
			 //----------  CONEXION A LA BD  ----------------
			
	         Class.forName("com.mysql.jdbc.Driver");
	         
	         //TODO controlar acceso identificado
	         conn = DriverManager.getConnection(GlobalData.url,GlobalData.login,GlobalData.password);

	         if (conn == null)
	         {
	            System.out.println("Conexin a base de datos "+GlobalData.url+" ... falhida");
	         }
	         
	         String variableID = null;
	         String lugarID = null;
	         
	         Node n = null;

	 		// Root element.
	 		Element root = xmldoc.getDocumentElement();
	 		n = root.getFirstChild();
	 		
	         //-------------------------------------------------------------
	 		do{
				Node ele = n.getFirstChild();
				CampoEnum campo = GlobalData.PrimerValorCampoEnum();
				
				do{
					switch (campo){
					
						case Variable:		//elemento: variable
							infoVariable[GlobalData.indexOfInfoVariable("NombreVariable")] = ele.getTextContent();
							break;
							
						case AditividadTemporal:		//elemento: aditividadTemporal
							infoVariable[GlobalData.indexOfInfoVariable("AditividadTemporal")] = ele.getTextContent();
							break;
							
						case AditividadEspacial:		//elemento: aditividadEspacial
							infoVariable[GlobalData.indexOfInfoVariable("AditividadEspacial")] = ele.getTextContent();
							break;
							
						case Frecuencia:		//elemento: frecuencia
							infoVariable[GlobalData.indexOfInfoVariable("Frecuencia")] = ele.getTextContent();
							break;
							
						case Fecha:		//elemento: fecha
							//TODO convertir fecha a float
							fecha = ele.getTextContent();
							break;
							
						case Unidades:		//elemento: unidades
							infoVariable[GlobalData.indexOfInfoVariable("Unidades")] = ele.getTextContent();
							break;
							
						case Lugar:		//elemento: lugar
							Cadena cad = new Cadena();
							String[] nom_aux = cad.obtenerNombresLugar(ele.getTextContent());
							lugar = nom_aux[0];
							break;
							
						case Nivel:		//elemento: nivel
							nivel = ele.getTextContent();
							
						case TipoDato:		//elemento: tipoDato
							infoVariable[GlobalData.indexOfInfoVariable("TipoDato")] = ele.getTextContent();
							break;
							
						case NaturalezaDato:		//elemento: naturalezaDato
							infoVariable[GlobalData.indexOfInfoVariable("NaturalezaDato")] = ele.getTextContent();
							break;
							
						case Valor:		//elemento: valor
							valor = ele.getTextContent();
							break;
							
						case OrigenDatos:		//elemento: OrigenDatos
							infoVariable[GlobalData.indexOfInfoVariable("OrigenDatos")] = ele.getTextContent();
							break;
						//TODO aqui se aaden los demas "case" en caso de modificar el diseo
					}

				campo = GlobalData.incrementarCampoEnum(campo);
				ele = ele.getNextSibling();
				
				}while (ele != null);
				
				 //----------  INSERTAMOS LOS DATOS OBTENIDOS EN LA BD  ---------------
				
				
		         //---  INSERCION DE TUPLA EN variables EN CASO DE SER NECESARIO  -----
		         //buscamos la variable en la tabla variables para obtener variableId
		         try{
		        	 Statement st = conn.createStatement();

		        	 String update = ("INSERT INTO variables (Nombre, AditividadTemporal, AditividadEspacial, Frecuencia, Unidades, TipoDato, NaturalezaDato, OrigenDatos)" +
		        			 "VALUES ('" + infoVariable[GlobalData.indexOfInfoVariable("NombreVariable")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("AditividadTemporal")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("AditividadEspacial")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("Frecuencia")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("Unidades")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("TipoDato")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("NaturalezaDato")] + 
		        			 "','" + infoVariable[GlobalData.indexOfInfoVariable("OrigenDatos")] + "')");
		        	 
		        	 st.executeUpdate(update);
		             
		         }catch(SQLException s){
		             if (!s.getMessage().contains("Duplicate entry")){
		            	 System.out.println("No se ha podido insertar la variable \"" + infoVariable[GlobalData.indexOfInfoVariable("NombreVariable")] + "\"");
		             }
		         }   
		         
		         //Obtenemos variableID de la variable en cuestion
		         try {
			         Statement st = conn.createStatement();
		             ResultSet res  = st.executeQuery("SELECT VariableID FROM `variables` " + 
		     				  "WHERE Nombre = '" + infoVariable[0] + "'");
	
		             if (res.next() == true){
			             variableID = res.getString("VariableID");
		             }
		         }catch(SQLException ex)
				 {
		        	 System.out.println("No se encuentra la variable \"" + infoVariable[0] + "\" en la Base de datos");
				 }

	             System.out.println(lugar);
	             try{
	            	 Statement st = conn.createStatement();
		             ResultSet res = st.executeQuery ("SELECT LugarID FROM `geografia` " + 
		    				  "WHERE Nombre = '" + lugar + "' AND Nivel = '" + nivel + "'");
		             
		             if (res.next() == true){
			             lugarID = res.getString("LugarId");
		             }
	             }catch(SQLException ex){
	            	 System.out.println("No se encuentra el lugar \"" + lugar + "(" + nivel + ")\" en la Base de datos");
	             }
	             
	             //---  INSERCION DE TUPLA EN datos  -----
	             //Insertamos la tupla con los datos en la tabla tupla
	             try{
	            	 Statement st = conn.createStatement();
		        	 String update = ("INSERT INTO `datos` (variableID, LugarID, fecha, valor)" +
		        			 "VALUES (" + variableID + "," + lugarID + "," + fecha + ",'" + valor + "')");
		        	 
		        	 st.executeUpdate(update);
	             }catch(SQLException ex){
		             if (!ex.getMessage().contains("Duplicate entry")){
		            	 System.out.println("No se han podido insertar los datos relativos a:");
		            	 System.out.println(infoVariable[GlobalData.indexOfInfoVariable("NombreVariable")] + "\t" + lugar + "\t" + fecha);
		             }else{
		            	 System.out.println("Ya existen en la Base de Datos los datos relativos a:");
		            	 System.out.println(infoVariable[0] + "\t" + lugar + "\t" + fecha);
		             }
	             }
	        	 
	        	 //---  FIN DE INSERCION DE TUPLA EN BD, ANALIZAMOS NUEVA TUPLA  --------- 
	        	 
				n = n.getNextSibling();
				
			} while (n != null);	
	         
	         //--------------------------------------------------------------
          
	         conn.close();
	         
	         //------- FIN CONEXION A LA BD  ----------------
	      }
		  catch(SQLException ex)
		  {
			  System.out.println("No se ha podido conectar con la Base de Datos");
			  System.out.println(ex);
		  }
		  catch(ClassNotFoundException ex)
		  {
			  System.out.println(ex);
		  }
	}
	
	//TODO pasar este main a otro archivo o hacerlo un metodo para guardar los 
	//datos de un directorio --> modificar cabecera args[0] = directorio
	public static void main( String args[] ) throws IOException{
		
	    FileReader fr = null;
	    BufferedReader br = null;

	    File carpeta = new File ("./datos_en_XML");
	    File archivos [] = null;
	    
	    if (carpeta.isDirectory()){
	    	archivos = carpeta.listFiles();
	    	
	    	for (int i=0; i<archivos.length; i++){
		        fr = new FileReader (archivos[i]);
		        br = new BufferedReader(fr);
		        
		        String linea = null, contenido = "";
		        while((linea=br.readLine())!=null){
		        	contenido = contenido + linea;
		        }
				
		        Document doc = String2Document(contenido);
		        introducirEnBDcamposString(doc);
	    	}
	    }
	}
}
